//@author: vvvv group
//@help: Effect processing for skinned mesh with directional light.
//@tags: skeleton, bones, collada, shading
//@credits: SlimDX

float4 LightDirection	= {0.0f, 0.0f, -1.0f, 1.0f};
float4 LightDiffuse <bool color = true;> = {0.6f, 0.6f, 0.6f, 1.0f};
float4 MaterialAmbient	<bool color = true;> = {0.1f, 0.1f, 0.1f, 1.0f};
float4 MaterialDiffuse	<bool color = true;> = {0.8f, 0.8f, 0.8f, 1.0f};

static const int MaxMatrices = 60;
float4x4 SkinningMatrices[MaxMatrices];

float4x4 tW: WORLD;
float4x4 tVP : VIEWPROJECTION;
float4x4 tWVP: WORLDVIEWPROJECTION;

//texture
Texture2D tex0 <string uiname="Texture";>;
SamplerState g_samLinear
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};

float4x4 tColor <string uiname="Color Transform";>;

struct VSInput
{
	float4 position			: POSITION;
	float4 normal		: NORMAL;
	float2 uv 			: TEXCOORD0;
	float4 BlendWeights	: BLENDWEIGHT;
	int4   BlendIndices	: BLENDINDICES;
};

struct VSOutput
{
	float4 position			: SV_POSITION;
	float2 uv			: TEXCOORD0;
	float4 Diffuse		: TEXCOORD1;
	

};

float3 Diffuse(float3 normal)
{
	float cosTheta;

	cosTheta = max(0.0f, dot(normal, -LightDirection.xyz));

	return (cosTheta);
}



VSOutput VS(VSInput input)
{
	VSOutput output = (VSOutput)0;

        /*
         * ---------- Skinning ----------
         */
	float4 blendWeights = input.BlendWeights;
	int4 indices = input.BlendIndices;
	
	float4 pos = 0;
	float3 norm = 0;

        for (int i = 0; i < 4; i++)
        {
        pos = pos + mul(input.position, SkinningMatrices[indices[i]]) * blendWeights[i];
	    norm = norm + mul(input.normal, SkinningMatrices[indices[i]]) * blendWeights[i];
        }
        /*
         * -------- End Skinning --------
         */

	norm = normalize(norm);

	output.position = mul(pos, tWVP);
	output.Diffuse.xyz = MaterialAmbient.xyz + Diffuse(norm) * MaterialDiffuse.xyz;
	output.Diffuse.w = 1.0f;
	output.uv = input.uv;

	return output;
}



float4 PS(VSOutput input) : SV_Target
{
	
   // float4 col = tex0.Sample(g_samLinear, input.TextureCoordinates.xy);
   	float4 col = 1;
	//col.rgb = input.Diffuse;
	//col.a = 1;
	
    return mul(col, tColor);
}

technique10 SkinnedMesh
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_4_0, VS() ) );
		SetPixelShader( CompileShader( ps_4_0, PS() ) );
	}
}
